
//
// cle4vdr 2005 - 2010 by Alexander Hans <cle4vdr@arcor.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
// Or, point your browser to http://www.gnu.org/copyleft/gpl.html
//


unit uchannels;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, ComCtrls, StrUtils, Dialogs;

type
    cleP = ^cleChannels;
    cleChannels = Record
    Number: Integer;
    GroupSep: Boolean;
    Name : String;
    Short: String;
    Provider: String;
    Source: String;
    Frequency: String;
    Pol: String;
    Symbolrate: String;
    Videopid: String;
    PcrPid: String;
    AudioPids: TStringList;
    AC3Pids: TStringList;
    Ttpid: String;
    Ca: String;
    Sid: String;
    Nid: String;
    Tid: String;
    Rid: String;
    Alpha: String;       // (0) 0, (1) ALPHA_1, (2) ALPHA_2, (4) ALPHA_4
    CodeRH: String;      // (0) none, (12) 1/2, (13) 1/3, (14) 1/4, (23) 2/3,
                         // (25) 2/5, (34) 3/4, (35) 3/5, 45 4/5, 56 5/6,
                         // (67) 6/7, (78) 7/8, (89) 8/9, 910 9/10, (999) AUTO
    CodeRL: String;
    Guard: String;       // (4) 1/4, (8) 1/8, (16) 1/16, (32) 1/32, 999 AUTO
    Modulation: String;  // (0)none, (4) QAM4, (16) QAM16, (32) QAM32, (64) QAM64,
                         // (128) QAM128, (256) QAM256, (512) QAM512, (1024) QAM1024,
                         // (1) BPSK, (2) QPSK, (3) OQPSK, (5) 8PSK, (6) 16APSK,
                         // (7) 32APSK, (8) OFDM, (9) COFDM, (10) VSB8,
                         // (11) VSB16, (998) QAMAUTO, (999)
    Priority: String;    // (0) high, (1) low
    Hierarchy: String;    // (0) off, (1) on, (999) AUTO
    Bandwidth: String;   // 5 MHz (5), 6 MHz(6), 7 MHz(7), 8 MHz(8), Auto(999)
    Inversion: String;   // on(1), off(0), auto(999)
    RollOff: String;     // (0) UNKNOWN, (20) 0.20, (25) 0.25, (35) 0.35,
    TransmissionMode: String;  // (2) 2K, (4) 4K, (8) 8K, (999) Auto
    System: String;      // (0), DVB-S, (1) DVB-S2
    IPTVParams: String;
  end;


var
  TVP: cleP;
  CopyData: array of cleP;

const
  CodeRHMap: array[0..14] of String =
             ('0', '12', '13','14', '23', '25', '34', '35', '45', '56', '67', '78', '89', '910', '999');
  GuardMap:  array[0..4] of String =
             ('4' , '8' , '16' , '32' , '999');
  ModulationMap: array[0..20] of String =
             ('0', '4', '16', '32', '64', '128', '256', '512', '1024', '1', '2', '3', '5', '6', '7', '8', '9', '10', '11', '998', '999');
  PriorityMap: array[0..1] of String =
              ('0', '1');
  HierarchyMap: array[0..2] of String =
              ('0', '1', '999');
  BandwidthMap: array[0..4] of String =
              ('5', '6', '7', '8', '999');
  InversionMap: array[0..2] of String =
              ('0', '1', '999');
  RolloffMap: array[0..3] of String =
              ('0', '20', '25', '35');
  TransmissionMap: array[0..3] of String =
              ('2', '4', '8', '999');
  SystemMap:  array[0..1] of String =
              ('0', '1');
  PolMap:     array[0..4] of String =
              ('h', 'v', 'l', 'r', 'p');
              
  NEWCHANNELSTR = 'New Channel:0:0:0:0:0:0:0:0:0:0:0:0';

procedure AssignNodeImage(n: TTreeNode);
procedure AddGroupSepStr(str: String);
function NewGroupSep(str: String): Pointer;
procedure AddChannelStr(str: String);
function NewChannel(str: String): Pointer;
procedure FillChannelData(str: String; p: cleP);
procedure InsertGroupSep(str: String);
procedure InsertChannel(str: String; n: TTreeNode; update: Boolean);
procedure InsertNewChannel(str: String; n: TTreeNode);

function LongChannelName(str: String): String;
function ShortChannelName(str: String): String;
function ProviderName(str: String): String;

function GetProviderStr(n: TTreeNode): String;
function GetChannelStr(n: TTreeNode; old: Boolean=false): String;
function AudioPidsToStr(n: TTreeNode): String;

function isTV(n: TTreeNode): Boolean;
function isRadio(n: TTreeNode): Boolean;
function isUnknown(n: TTreeNode): Boolean;
function isGroupSep(n: TTreeNode): Boolean;
function isEncrypted(n: TTreeNode): Boolean;
function ChannelType(n: TTreeNode): String;
function ChanIDFreq(pol, freq: String): String;
function ChannelID(n: TTreeNode): String;
function LanguagesToStr(n: TTreeNode): String;
function HasDolby(n: TTreeNode): Boolean;
function HasAudio(n: TTreeNode): Boolean;
function HasMultiLang(n: TTreeNode): Boolean;

function GetAudioPid(str: String): String;
function GetLanguage(str: String): String;
function GetVidPid(str: String): String;
function GetPcrPid(str: String): String;
function GetParam(str: String; c: Char): String;
function GetAudioPids(str: String): TStringList;
function GetAC3Pids(str: String): TStringList;
function PolToStr(str: String): String;
function EncryptionsToStr(n: TTreeNode): String;

procedure RenumberChannels;
function GetPrevChannel(n: TTreeNode): TTreeNode;
function GetNextChannel(n: TTreeNode): TTreeNode;
function GetNextGroupSep(n: TTreeNode): TTreeNode;
function HasWrongGroupSep(n: TTreeNode): Boolean;
function GetRightGroupSep(n: TTreeNode): TTreeNode;

function GetAllSources(var sl: TStringList): Integer;
procedure RemoveSources(source: String);
procedure RemoveAll(target: String);

implementation

uses uinterface, uutils, uinfo, uedit;

procedure AssignNodeImage(n: TTreeNode);
begin
  if n = NIL then Exit;
  if isGroupSep(n) then n.ImageIndex := 0
  else if isTV(n) then
  begin
    if isEncrypted(n) then n.ImageIndex := 2
    else n.ImageIndex := 1;
  end
  else if isRadio(n) then
  begin
    if isEncrypted(n) then n.ImageIndex := 4
    else n.ImageIndex := 3;
  end
  else n.ImageIndex := 5;
  n.SelectedIndex := n.ImageIndex;
end;


procedure AddGroupSepStr(str: String);
begin
  if length(str) < 1 then Exit;
  str := copy(str, 2, length(str));
  str := AnsiReplaceStr(str, '|', ':');
  Node := frmMain.tvMain.Items.GetLastNode;
  Node := frmMain.tvMain.Items.AddObject(Node, str, NewGroupSep(str));
  AssignNodeImage(Node);
end;

function NewGroupSep(str: String): Pointer;
begin
  New(TVP);
  result           := TVP;
  TVP^.Number      := 0;
  TVP^.GroupSep    := true;
  TVP^.Name        := str;
  TVP^.Short       := '';
  TVP^.Provider    := '';
  TVP^.Source      := '';
  TVP^.Frequency   := '';
  TVP^.Pol         := '';
  TVP^.Symbolrate  := '';
  TVP^.Videopid    := '';
  TVP^.PcrPid      := '';
  TVP^.AudioPids   := NIL;
  TVP^.AC3Pids     := NIL;
  TVP^.TTpid       := '';
  TVP^.Ca          := '';
  TVP^.Sid         := '';
  TVP^.Nid         := '';
  TVP^.Tid         := '';
  TVP^.Rid         := '';
  TVP^.Alpha       := '';
  TVP^.CodeRH      := '';
  TVP^.CodeRL      := '';
  TVP^.Guard       := '';
  TVP^.Modulation  := '';
  TVP^.Priority    := '';
  TVP^.Hierarchy    := '';
  TVP^.Bandwidth   := '';
  TVP^.Inversion   := '';
  TVP^.RollOff     := '';
  TVP^.TransmissionMode := '';
  TVP^.System      := '';
  TVP^.IPTVParams  := '';
end;

procedure AddChannelStr(str: String);
var
  dummy: String;
  n: TTreeNode;
begin
  dummy := str;
  str := copy(str, 1, AnsiPos(':', str)-1);
  str := AnsiReplaceStr(str, '|', ':');
  if (frmMain.tvMain.Items.Count > 0) AND
     ((isGroupSep(frmMain.tvMain.Items.GetLastNode)) OR (frmMain.tvMain.Items.GetLastNode.Parent <> NIL)) then
    n := frmMain.tvMain.Items.AddChildObject(Node, LongChannelName(str), NewChannel(dummy))
  else
    n := frmMain.tvMain.Items.AddObject(Node, LongChannelName(str), NewChannel(dummy));
  AssignNodeImage(n);
end;

function NewChannel(str: String): Pointer;
begin
  New(TVP);
  result := TVP;
  FillChannelData(str, TVP);
end;

procedure FillChannelData(str: String; p: cleP);
var
  strl: TStringList;
begin
  strl := StringSplit(str, ':');
  if strl.Count > 10 then
  begin
    p^.Number     := 0;
    p^.GroupSep   := false;
    strl[0] := AnsiReplaceStr(strl[0], '|', ':');
    p^.Name       := LongChannelName(strl[0]);
    p^.Short      := ShortChannelName(strl[0]);
    p^.Provider   := ProviderName(strl[0]);
    p^.Frequency  := strl[1];
    if strl[2] <> '' then
    begin
      if UpperCase(strl[3]) = 'P' then p^.Pol := strl[2]
      else p^.Pol := strl[2][1]
    end
    else p^.Pol := '';
    p^.Source     := UpperCase(strl[3]);
    p^.Symbolrate := strl[4];
    p^.Videopid   := GetVidPid(strl[5]);
    p^.PcrPid     := GetPcrPid(strl[5]);
    p^.AudioPids  := GetAudioPids(strl[6]);
    p^.AC3Pids    := GetAC3Pids(strl[6]);
    p^.TTpid      := strl[7];
    p^.Ca         := strl[8];
    p^.Sid        := strl[9];
    p^.Nid        := strl[10];
    p^.Tid        := strl[11];
    if strl.Count > 12 then
      p^.Rid        := strl[12]
    else p^.Rid := '0';
    if p^.Source = 'P' then p^.IPTVParams := strl[2]
    else
    begin
      p^.Alpha      := GetParam(strl[2], 'A');
      p^.CodeRH     := GetParam(strl[2], 'C');
      p^.CodeRL     := GetParam(strl[2], 'D');
      p^.Guard      := GetParam(strl[2], 'G');
      p^.Modulation := GetParam(strl[2], 'M');
      p^.Priority   := GetParam(strl[2], 'P');
      p^.Hierarchy  := GetParam(strl[2], 'Y');
      p^.Bandwidth  := GetParam(strl[2], 'B');
      p^.Inversion  := GetParam(strl[2], 'I');
      p^.RollOff    := GetParam(strl[2], 'O');
      p^.TransmissionMode := GetParam(strl[2], 'T');
      p^.System     := GetParam(strl[2], 'S');
      try
        if (p^.Source[1] = 'S') AND (p^.System = '') then p^.System := '0';
      except
      end;
    end;
  end;
  strl.Free;
end;


procedure InsertGroupSep(str: String);
var
  n, tn: TTreeNode;
  i, j: Integer;
begin
  n := frmMain.tvMain.Items.InsertObject(frmMain.tvMain.Selected, str, NewGroupSep(str));
  frmMain.tvMain.Selected := n;
  frmMain.LastTargetNode := n;
  RenumberChannels;
  if frmMain.pcMain.ActivePage = frmMain.tsInfo then
  begin
    frmMain.edName.SetFocus;
    frmMain.edName.SelectAll;
  end;
end;


procedure InsertChannel(str: String; n: TTreeNode; update: Boolean);
var
  name: String;
begin
  name := copy(str, 1, AnsiPos(':', str)-1);
  name := AnsiReplaceStr(name, '|', ':');
  if isGroupSep(n) then
    n := frmMain.tvMain.Items.AddChildObject(n, name, NewChannel(str))
  else
    n := frmMain.tvMain.Items.InsertObject(n, name, NewChannel(str));
  AssignNodeImage(n);
  if update then
  begin
    RenumberChannels;
    FileChanged(true);
    frmMain.tvMain.Selected := n;
    if frmMain.pcMain.ActivePage = frmMain.tsInfo then
    begin
      frmMain.edName.SetFocus;
      frmMain.edName.SelectAll;
    end;
  end;
end;

procedure InsertNewChannel(str: String; n: TTreeNode);
var
  source, pol: String;
begin
  source := '';
  if isGroupSep(n) AND (n.HasChildren) then
  begin
    source := cleP(n.Items[0].Data)^.Source;
    n := frmMain.tvMain.Items.AddChildObject(n, 'New Channel', NewChannel(str));
  end
  else if (isGroupSep(n)) AND (NOT n.HasChildren) then n := frmMain.tvMain.Items.AddChildObject(n, 'New Channel', NewChannel(str))  //TODO
  else
  begin
    source := cleP(n.Data)^.Source;
    pol    := cleP(n.Data)^.Pol;
    n := frmMain.tvMain.Items.InsertObject(n, 'New Channel', NewChannel(str));
  end;
  cleP(n.Data)^.Source := source;
  cleP(n.Data)^.Pol    := pol;
  RenumberChannels;
  FileChanged(true);
  frmMain.tvMain.Selected := n;
  if frmMain.pcMain.ActivePage = frmMain.tsInfo then
  begin
    frmMain.edName.SetFocus;
    frmMain.edName.SelectAll;
  end;
end;


// ********* String Helpers ***********//

function LongChannelName(str: String): String;
begin
  if AnsiContainsText(str, ';') then str := copy(str, 1, AnsiPos(';', str)-1);
  if AnsiContainsText(str, ',') then str := copy(str, 1, RPos(',', str)-1);
  result := str;
end;

function ShortChannelName(str: String): String;
begin
  if AnsiContainsText(str, ';') then str := copy(str, 1, AnsiPos(';', str)-1);
  if AnsiContainsText(str, ',') then str := copy(str, RPos(',', str)+1, length(str))
  else  str := '';
  result := AnsiReplaceStr(str, '.', ',');
end;

function ProviderName(str: String): String;
begin
  if AnsiContainsText(str, ';') then result := copy(str, AnsiPos(';', str)+1, length(str))
  else result := '';
end;


// ********** Saving ***************//

function GetProviderStr(n: TTreeNode): String;
begin
  result := ':' + AnsiReplaceStr(n.Text, ':', '|');
end;


function GetParametrStr(n: TTreeNode): String;
  function isPolStr(s: String): Boolean;
  var
    i: Integer;
  begin
    result := true;
    if s = '' then exit;
    for i := 0 to High(PolMap) do
      if LowerCase(s[1]) = PolMap[i] then exit;
    result := false;
  end;
begin
  result := '';
  with cleP(n.Data)^do
  begin
    if Source = '' then Exit;
    if Source[1] in ['S']             then if isPolStr(Pol)          then result += Pol;
    if Source[1] in ['T']             then if Alpha <> ''            then result += 'A' + Alpha;
    if Source[1] in ['T']             then if Bandwidth <> ''        then result += 'B' + Bandwidth;
    if Source[1] in ['C','S','T']     then if CodeRH <> ''           then result += 'C' + CodeRH;
    if Source[1] in ['T']             then if CodeRL <> ''           then result += 'D' + CodeRL;
    if Source[1] in ['T']             then if Guard <> ''            then result += 'G' + Guard;
    if Source[1] in ['A','C','S','T'] then if Inversion <> ''        then result += 'I' + Inversion;
    if Source[1] in ['A','C','S','T'] then if Modulation <> ''       then result += 'M' + Modulation;
    if Source[1] in ['S']             then if RollOff <> ''          then result += 'O' + RollOff;
    if Source[1] in ['T']             then if Priority <> ''         then result += 'P' + Priority;
    if Source[1] in ['S']             then if System <> ''           then result += 'S' + System;
    if Source[1] in ['T']             then if TransmissionMode <> '' then result += 'T' + TransmissionMode;
    if Source[1] in ['T']             then if Hierarchy <> ''        then result += 'Y' + Hierarchy;
    if Source[1] in ['P']             then result := IPTVParams;
  end;
end;

function GetOldParametrStr(n: TTreeNode): String;
  function isPolStr(s: String): Boolean;
  var
    i: Integer;
  begin
    result := true;
    if s = '' then exit;
    for i := 0 to High(PolMap) do
      if s[1] = PolMap[i] then exit;
    result := false;
  end;
begin
  result := '';
  with cleP(n.Data)^do
  begin
    if Source = '' then Exit;
    if Source[1] in ['S']         then if isPolStr(Pol)          then result += Pol;
    if Source[1] in ['C','S','T'] then if Inversion <> ''        then result += 'I' + Inversion;
    if Source[1] in ['C','S','T'] then if CodeRH <> ''           then result += 'C' + CodeRH;
    if Source[1] in ['T']         then if CodeRL <> ''           then result += 'D' + CodeRL;
    if Source[1] in ['C','T']     then if Modulation <> ''       then result += 'M' + Modulation;
    if Source[1] in ['T']         then if Bandwidth <> ''        then result += 'B' + Bandwidth;
    if Source[1] in ['T']         then if TransmissionMode <> '' then result += 'T' + TransmissionMode;
    if Source[1] in ['T']         then if Guard <> ''            then result += 'G' + Guard;
    if Source[1] in ['T']         then if Hierarchy <> ''        then result += 'Y' + Hierarchy;
    if Source[1] in ['P']         then result := IPTVParams;
  end;
end;


function GetChannelStr(n: TTreeNode; old: Boolean=false): String;
begin
  result     := AnsiReplaceStr(cleP(n.data)^.Name, ':', '|');
  if cleP(n.data)^.Short <>'' then
  begin
    cleP(n.data)^.Short := AnsiReplaceStr(cleP(n.data)^.Short, ':', '|');
    cleP(n.data)^.Short := AnsiReplaceStr(cleP(n.data)^.Short, ',', '.');
    result := result + ',' + cleP(n.data)^.Short;
  end;
  if cleP(n.data)^.Provider <>'' then
  result := result + ';' + AnsiReplaceStr(cleP(n.data)^.Provider, ':', '|');
  result     := result + ':';
  result     := result + cleP(n.data)^.Frequency + ':';
  if old then result := result + GetOldParametrStr(n) + ':'
  else result := result + GetParametrStr(n) + ':';
  result     := result + cleP(n.data)^.Source + ':';
  result     := result + cleP(n.data)^.Symbolrate + ':';
  result     := result + cleP(n.data)^.Videopid;
  if (cleP(n.data)^.PcrPid <> '0') AND (cleP(n.data)^.PcrPid <> '') AND
     (cleP(n.data)^.PcrPid <> cleP(n.data)^.Videopid) then
                result += '+' + cleP(n.data)^.PcrPid + ':'
  else result += ':';
  result     := result + AudioPidsToStr(n) + ':';
  result     := result + cleP(n.data)^.TTpid + ':';
  result     := result + cleP(n.data)^.Ca + ':';
  result     := result + cleP(n.data)^.Sid + ':';
  result     := result + cleP(n.data)^.Nid + ':';
  result     := result + cleP(n.data)^.Tid + ':';
  result     := result + cleP(n.data)^.Rid;
end;


function AudioPidsToStr(n: TTreeNode): String;
var
  i: Integer;
begin
  result := '';
  if (cleP(n.Data)^.AudioPids <> NIL) AND (cleP(n.Data)^.AudioPids.Count > 0) then
  begin
    for i := 0 to cleP(n.Data)^.AudioPids.Count - 1 do
      result += cleP(n.Data)^.AudioPids[i] + ',';
    if length(result) > 0 then result := copy(result, 1, length(result)-1);
  end;
  if (cleP(n.Data)^.AC3Pids <> NIL) AND (cleP(n.Data)^.AC3Pids.Count > 0) then
  begin
    result += ';';
    for i := 0 to cleP(n.Data)^.AC3Pids.Count - 1 do
      result += cleP(n.Data)^.AC3Pids[i] + ',';
    result := RTrunc(result, 1);
  end;
end;

// ******** Channel properties (TreeNode) **********//

function isTV(n: TTreeNode): Boolean;
begin
  result := (NOT isGroupSep(n)) AND (cleP(n.Data)^.VideoPid <> '0');
end;

function isRadio(n: TTreeNode): Boolean;
begin
  result := (NOT isGroupSep(n)) AND (cleP(n.Data)^.VideoPid <= '1') AND (HasAudio(n));
end;

function isUnknown(n: TTreeNode): Boolean;
begin
  result := (NOT isGroupSep(n)) AND (NOT isTV(n)) AND (NOT isRadio(n));
end;

function isGroupSep(n: TTreeNode): Boolean;
begin
  result := cleP(n.Data)^.GroupSep;
end;

function isIPTV(n: TTreeNode): Boolean;
begin
  result := cleP(n.Data)^.Source = 'P';
end;

function isAnalog(n: TTreeNode): Boolean;
begin
  result := cleP(n.Data)^.Ca = 'A0'
end;

function isEncrypted(n: TTreeNode): Boolean;
begin
  result := (cleP(n.Data)^.Ca <> '0') AND (cleP(n.Data)^.Ca <> '') AND (cleP(n.Data)^.Ca <> 'A0')
end;

function ChannelType(n: TTreeNode): String;
begin
  result := '';
  if isGroupSep(n) then result := 'Group'
  else if isTV(n) then result := 'Television'
  else if isRadio(n) then result := 'Radio'
  else if isUnknown(n) then result := 'Data'
  else if isIPTV(n) then result := 'IPTV'
  else if isAnalog(n) then result := 'Analog'
end;


function ChanIDFreq(pol, freq: String): String;
var
  dummy: Integer;
begin
  result := freq;
  if (length(pol) > 0) AND (UpperCase(pol)[1] in ['H', 'V', 'L', 'R']) then
  begin
    Try
      dummy := StrToInt(freq);
    Except on eConvertError do exit;
    end;
    if UpperCase(pol[1]) = 'H' then dummy += 100000
    else if UpperCase(pol[1]) = 'V' then dummy += 200000
    else if UpperCase(pol[1]) = 'L' then dummy += 300000
    else if UpperCase(pol[1]) = 'R' then dummy += 400000;
    result := IntToStr(dummy);
  end;
end;

function ChannelID(n: TTreeNode): String;
begin
  result := '';
  result += clep(n.Data)^.Source + '-';
  result += clep(n.Data)^.NID + '-';
  if (cleP(n.Data)^.Tid = '0') AND (cleP(n.Data)^.NID = '0') then
    result += ChanIdFreq(clep(n.Data)^.Pol, clep(n.Data)^.Frequency) + '-'
  else result += clep(n.Data)^.Tid + '-';
  result += clep(n.Data)^.SID + '-';
  result += clep(n.Data)^.Rid;
end;

function LanguagesToStr(n: TTreeNode): String;
var
  i: Integer;
  dummy: String;
begin
  result := '';
  dummy := '';
  if (cleP(n.Data)^.AudioPids = NIL) OR (cleP(n.Data)^.AudioPids.Count = 0) then Exit;
  for i := 0 to cleP(n.Data)^.AudioPids.Count-1 do
  begin
    if cleP(n.Data)^.AudioPids[i] = '0' then continue;
    dummy := GetLanguage(cleP(n.Data)^.AudioPids[i]);
    if dummy <> '' then result += dummy + ', ';
  end;
  result := RTrunc(result, 2);
end;

function HasDolby(n: TTreeNode): Boolean;
begin
  if (cleP(n.Data)^.AC3Pids <> NIL) AND (cleP(n.Data)^.AC3Pids.Count > 0) then result := true
  else result := false;
end;

function HasAudio(n: TTreeNode): Boolean;
begin
  if (cleP(n.Data)^.AudioPids <> NIL) AND (cleP(n.Data)^.AudioPids.Count > 0) AND
     (cleP(n.Data)^.AudioPids[0] <> '0') then result := true
  else result := false;
end;

function HasMultiLang(n: TTreeNode): Boolean;
begin
  if (cleP(n.Data)^.AudioPids <> NIL) AND (cleP(n.Data)^.AudioPids.Count > 1) then result := true
  else result := false;
end;



// ******** Channel properties (String) **********//


function GetAudioPid(str: String): String;
begin
  if AnsiContainsText(str, '=') then
     result := copy(str, 1, AnsiPos('=', str) - 1)
  else result := str;
end;

function GetLanguage(str: String): String;
begin
  if AnsiContainsText(str, '=') then
     result := copy(str, AnsiPos('=', str) + 1, length(str))
  else result := '';
end;

function GetVidPid(str: String): String;
begin
  if AnsiContainsText(str, '+') then
    result := copy(str, 1, AnsiPos('+', str)-1)
  else result := str;
end;

function GetPcrPid(str: String): String;
begin
  if str = '0' then result := '';
  if AnsiContainsText(str, '+') then
    result := copy(str, AnsiPos('+', str)+1, length(str))
  else result := '0';
end;

function GetParam(str: String; c: Char): String;
var
  i: Integer;
begin
  result := '';
  i := Pos(c, str);
  if i > 0 then
  begin
    While (i < Length(str)) AND (str[i+1] in ['0'..'9']) do
    begin
      result += str[i+1];
      inc(i);
    end;
  end;
end;

function GetAudioPids(str: String): TStringList;
begin
  if AnsiContainsStr(str, ';') then str := copy(str, 1, Pos(';', str) -1);
  result := StringSplit(str, ',');
end;

function GetAC3Pids(str: String): TStringList;
begin
  if AnsiContainsStr(str, ';') then str := copy(str, Pos(';', str) + 1, length(str))
  else
  begin
    result := TStringList.Create;
    exit;
  end;
  result := StringSplit(str, ',');
end;

function PolToStr(str: String): String;
begin
  result := '';
  if length(str) < 1 then Exit;
  Case UpperCase(str)[1] of
          'H': result := 'Horizontal';
          'V': result := 'Vertical';
          'L': result := 'Left Circular';
          'R': result := 'Right Circular';
  end;
end;

function EncryptionsToStr(n: TTreeNode): String;
var
  CaSystems, ca: TStringList;
  i, d: Integer;
begin
  result := '';
  if cleP(n.Data)^.Ca = '0' then Exit;
  CaSystems := StringSplit(cleP(n.Data)^.Ca, ',');
  ca := TStringList.Create;
  ca.Sorted := true;
  ca.Duplicates := dupIgnore;
  for i := 0 to CaSystems.Count - 1 do
  begin
    if NOT TryStrToInt('$'+CaSystems[i], d) then continue;
    if d >= 256 then
    begin
      d := d DIV 256;
      case d of
         1:  ca.Add('Seca/Mediagard');
         5:  ca.Add('Viaccess');
         6:  ca.Add('Irdeto');
         9:  ca.Add('NDS/Videoguard');
        11:  ca.Add('Conax');
        13:  ca.Add('CryptoWorks');
        14:  ca.Add('PowerVu');
        18:  ca.Add('NagraVision');
        23:  ca.Add('BetaCrypt');
        24:  ca.Add('NagraVision');
        74:  ca.Add('SkyCrypt');
      end
    end
    else if (d >= 16) AND (d <= 255) then
    begin
      if d = 160 then ca.Add('TV-analog')
      else ca.Add('User defined')
    end
    else if (d >= 1) AND (d <= 15) then ca.Add('DVB'+ CaSystems[i]);
  end;
  ca.Sort;
  for i := 0 to ca.Count - 1 do result += ca[i] + ', ';
  result := RTrunc(result, 2);
  CaSystems.Free;
  Ca.Free;
end;


// *********** TreeView Helpers ************//

procedure RenumberChannels;
var
  i, j, r: Integer;
begin
  j := 0;
  r := 0;
  For i := 0 to frmMain.tvMain.Items.Count-1 do
  begin
    if (isGroupSep(frmMain.tvMain.Items[i])) AND (isJumpMark(frmMain.tvMain.Items[i]))
            then
            if (GetPrevChannel(frmMain.tvMain.Items[i]) <> NIL) AND
            (cleP(GetPrevChannel(frmMain.tvMain.Items[i]).Data)^.Number < GetJumpIndex(frmMain.tvMain.Items[i])) then
            j := GetJumpIndex(frmMain.tvMain.Items[i])-1;
    if NOT isGroupSep(frmMain.tvMain.Items[i]) then
    begin
      inc(j);
      inc(r);
      cleP(frmMain.tvMain.Items[i].Data)^.Number := j;
    end
    else cleP(frmMain.tvMain.Items[i].Data)^.Number := 0;
  end;
  sbm(GetChannelsInfo, 1);
end;

function GetPrevChannel(n: TTreeNode): TTreeNode;
begin
  result := n;
  While (result <> NIL) AND (isGroupSep(result)) do result := result.GetPrev;
end;

function GetNextChannel(n: TTreeNode): TTreeNode;
begin
  result := n;
  While (result <> NIL) AND (isGroupSep(result)) do result := result.GetNext;
end;

function GetNextGroupSep(n: TTreeNode): TTreeNode;
var
  tn: TTreeNode;
begin
  result := NIL;
  tn := n.GetNext;
  While tn <> NIL do
  begin
    if isGroupSep(tn) then
    begin
      result := tn;
      exit;
    end;
    tn := tn.GetNext;
  end;
end;

function HasWrongGroupSep(n: TTreeNode): Boolean;
var
  i: Integer;
begin
  result := false;
  if isGroupSep(n) then Exit;
  for i := 0 to frmMain.tvMain.Items.Count -1 do
  begin
    if (NOT isGroupSep(frmMain.tvMain.Items[i])) OR (frmMain.tvMain.Items[i].HasChildren = false) then continue;
    if (frmMain.tvMain.Items[i].Text = cleP(n.Data)^.Provider) AND
       (cleP(frmMain.tvMain.Items[i+1].data)^.Source = cleP(n.Data)^.Source) AND
       (ChannelType(frmMain.tvMain.Items[i+1]) = ChannelType(n)) AND
       (n.HasAsParent(frmMain.tvMain.Items[i]) = false) then
    begin
      result := true;
      Exit;
    end;
  end;
end;

function GetRightGroupSep(n: TTreeNode): TTreeNode;
var
  i: Integer;
begin
  result := NIL;
  if isGroupSep(n) then Exit;
  for i := 0 to frmMain.tvMain.Items.Count -1 do
  begin
    if (NOT isGroupSep(frmMain.tvMain.Items[i]))  OR (frmMain.tvMain.Items[i].HasChildren = false) then continue;
    if (frmMain.tvMain.Items[i].Text = cleP(n.Data)^.Provider) AND
       (cleP(frmMain.tvMain.Items[i+1].data)^.Source = cleP(n.Data)^.Source) AND
       (ChannelType(frmMain.tvMain.Items[i+1]) = ChannelType(n)) then
    begin
      result := frmMain.tvMain.Items[i];
      Exit;
    end;
  end;
end;


function GetAllSources(var sl: TStringList): Integer;
var
  i: Integer;
begin
  result := 0;
  if frmMain.tvMain.Items.Count = 0 then exit;
  for i := 0 to frmMain.tvMain.Items.Count-1 do
  begin
    if (isGroupSep(frmMain.tvMain.Items[i])) OR
       (trim(cleP(frmMain.tvMain.Items[i].Data)^.Source) = '') then continue;
    if sl.IndexOf(cleP(frmMain.tvMain.Items[i].Data)^.Source) < 0 then
      sl.Add(cleP(frmMain.tvMain.Items[i].Data)^.Source)
  end;
  result := sl.Count;
end;

procedure RemoveSources(source: String);
var
  i: Integer;
  tp, t: TTreeNode;
begin
  i := frmMain.tvMain.Items.Count - 1;
  While i > 0 do
  begin
    if cleP(frmMain.tvMain.Items[i].Data)^.Source = source then
    begin
      tp := frmMain.tvMain.Items[i].Parent;
      DeleteNode(frmMain.tvMain, frmMain.tvMain.Items[i], false);
      if (tp <> NIL) AND
         (cleP(tp.Data)^.GroupSep = true) AND
         (tp.HasChildren = false) then
         begin
           DeleteNode(frmMain.tvMain, tp, false);
           dec(i);
         end;
    end;
    dec(i);
  end;
  RenumberChannels;
  FileChanged(true);
end;

procedure RemoveAll(target: String);
var
  i: Integer;
  tp, t: TTreeNode;
begin
  i := frmMain.tvMain.Items.Count - 1;
  While i > 0 do
  begin
    if ((target = 'TV') AND (isTV(frmMain.tvMain.Items[i]))) OR
       ((target = 'Radio') AND (isRadio(frmMain.tvMain.Items[i]))) OR
       ((target = 'Data') AND (isUnknown(frmMain.tvMain.Items[i]))) OR
       ((target = 'Encrypted') AND (isEncrypted(frmMain.tvMain.Items[i])))   then
    begin
      tp := frmMain.tvMain.Items[i].Parent;
      DeleteNode(frmMain.tvMain, frmMain.tvMain.Items[i], false);
      if (tp <> NIL) AND
         (cleP(tp.Data)^.GroupSep = true) AND
         (tp.HasChildren = false) then
         begin
           DeleteNode(frmMain.tvMain, tp, false);
           dec(i);
         end;
    end;
    dec(i);
  end;
  RenumberChannels;
  FileChanged(true);
end;

end.

