
//
// cle4vdr 2005 - 2010 by Alexander Hans <cle4vdr@arcor.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
// Or, point your browser to http://www.gnu.org/copyleft/gpl.html
//


unit uiokaffeine;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StrUtils, Dialogs
{$IFDEF UNIX}
  ,BaseUnix, Dos
{$ENDIF};


const
  MAXAPIDS=20;
  KAFFEINEHEADER='#Generated by Kaffeine 0.5';
  KAFFEINEDIR='~/.kde/share/apps/kaffeine/dvb-s';

var
  SatNames, ChanNames: TStringList;

procedure WriteKaffeine(path: String);

implementation

uses uinterface, uchannels, uutils;

procedure FillSatNames;
begin
  SatNames := TStringList.Create;
  SatNames.Add('19.2E=Astra');
  SatNames.Add('13.0E=Hotbird');
  SatNames.Add('103w=AMC1');
  SatNames.Add('85w=AMC2');
  SatNames.Add('87w=AMC3');
  SatNames.Add('101w=AMC4');
  SatNames.Add('79w=AMC5');
  SatNames.Add('72w=AMC6');
  SatNames.Add('83w=AMC9');
  SatNames.Add('105.5E=AsiaSat3S');
  SatNames.Add('28.2E=Astra');
  SatNames.Add('28.5E=Eurobird1');
  SatNames.Add('123w=Galaxy10R');
  SatNames.Add('91w=Galaxy11');
  SatNames.Add('95w=Galaxy3C');
  SatNames.Add('30.0W=Hispasat');
  SatNames.Add('97w=IA5');
  SatNames.Add('93w=IA6');
  SatNames.Add('129w=IA7');
  SatNames.Add('89w=IA8');
  SatNames.Add('156E=OptusC1');
  SatNames.Add('43.0W=PAS');
  SatNames.Add('5.0E=Sirius');
  SatNames.Add('8.0W=Telecom2');
  SatNames.Add('15.0W=Telstar12');
  SatNames.Add('1.0W=Thor');
  SatNames.Add('42.0E=Turksat');

end;

function VToKAPid(aud: TStringList; ac3: TStringList): String;
var
  dummy2: String;
  apids: TStringList;
  i: Integer;
begin
  dummy2 := '';
  result := '';
  apids := TStringList.Create;


  for i := 0 to aud.Count -1 do
  begin
    if AnsiContainsStr(aud[i], '=') then
      dummy2 := copy(aud[i], 1, Pos('=', aud[i])-1) + '(' +
                copy(aud[i], Pos('=', aud[i])+1, 3) + ')'
    else dummy2 := aud[i];
    apids.Add(dummy2);
  end;

  if ac3.Count <> 0 then
  begin
    for i := 0 to ac3.Count-1 do
    begin
      if AnsiContainsStr(ac3[i], '=') then
        apids.Add(copy(ac3[i], 1, Pos('=', ac3[i])-1) + '(ac3)')
      else
        apids.Add(ac3[i] + '(ac3)');
    end;
  end;
  if apids.Count > MAXAPIDS then
    for i := apids.Count-1 downto MAXAPIDS do
      apids.Delete(i);
  for i := 0 to apids.Count - 1 do result += apids[i] + ',';
  apids.Free;

end;


{$IFDEF UNIX}
function KaffeineSatName(position: String): String;
var
  KDir: PDir;
  KDirent: PDirent;
  dummy: String;
begin
  KDir := fpOpenDir(FExpand(KAFFEINEDIR));
  if KDir = NIL then exit;
  Repeat
    KDirent := fpReadDir(KDir^);
    if KDirent <> NIL then
    begin
      dummy := Strpas(@KDirent^.d_name[0]);
      if (AnsiContainsStr(dummy, position)) AND NOT (AnsiContainsStr(dummy, '~')) then
      begin
        result := copy(dummy, 1, Pos('-', dummy)-1);
        SatNames.Add(position + '=' + result);
        break;
      end;
    end;
  until KDirent = NIL;
end;
{$ENDIF}
{$IFDEF WINDOWS}
function KaffeineSatName(position: String): String;
begin
  result := '';
end;
{$ENDIF}

function GetSatNameFromUser(position: String): String;
begin
  result := InputBox('no satellite name found on position '+ position, 'name for sat on '+ position, '');
  SatNames.Add(position + '=' + IfThen(result = '', '?', result));
  result += IfThen(result = '', '-?', '-');
end;


function GetSatName(position: String): String;
begin
  result := SatNames.Values[position]+'-';
  if result = '-' then
    //Satname not found in pre-defines
    //looking for file in KAFFEINEDIR (Linux only)
    result := KaffeineSatName(position);
  if result = '' then
    //asking the user for satname
    result := GetSatNameFromUser(position);

  if result = '-?' then result := '';
  result := 'S' + result + position;
end;



function AddSatName(s: String): String;
begin
  result := s;
  if (s = '') OR (UpperCase(s[1]) <> 'S') then exit;
  s := copy(s, 2, length(s)-1);
  result := GetSatName(s);
end;

function UniqueChannelName(s: String): String;
var
  i: Integer;
begin
  result := s;
  i := 0;
  While ChanNames.IndexOf(result) > -1 do
  begin
    inc(i);
    result := s + ' - (' + IntToStr(i) + ')';
  end;
  ChanNames.Add(result);
end;


procedure WriteKaffeine(path: String);
var
  outf: TextFile;
  i: Integer;
  dummy: String;
begin
  Try
    AssignFile(outf, path);
    Rewrite(outf);
  Except
    begin
      ShowMessage('Could not create ' + path); //TODO
      Exit;
    end;
  end;
  ChanNames := TStringList.Create;
  ChanNames.CaseSensitive := true;
  Writeln(outf, KaffeineHeader);
  For i := 0 to frmMain.tvMain.Items.Count - 1 do
  begin
    with cleP(frmMain.tvMain.Items[i].Data)^ do
    begin
      if GroupSep then Continue;
      dummy := '';
      dummy := IfThen(Videopid <> '0', 'TV', 'Rad');
      dummy += IfThen(Ca <> '0', 'C|', '|');
      dummy += UniqueChannelName(Name) + '|';
      dummy += GetVidPid(Videopid) + IfThen(GetVidPid(Videopid) <> '0', '(2)|', '|');
      dummy += VToKAPid(AudioPids, AC3Pids) + '|';
      dummy += TTPid + '|';
      dummy += Sid + '|';
      dummy += Tid + '|';
      Case UpperCase(Source)[1] of
        'S' : dummy += AddSatName(Source) + '|';
              // TODO else dummy += GetSatNameFromUser(Source) + '|';
        'C' : dummy += 'Cable|';
        'T' : dummy += 'Terrestrial|';
      end;
      dummy += Frequency + '|';
      dummy += Symbolrate + '|';
      if UpperCase(Source)[1] = 'S' then
      begin
        dummy += UpperCase(Pol)[1] + '|';
        dummy += IfThen(length(Pol) > 2, copy(Pol, 3, 2), ' ')  + '|';
        dummy += '-1|-1|-1|-1|-1|-1|-1|'
      end
      else
      begin
        dummy += '||';
        dummy += CodeRH            + '|';
        dummy += Inversion         + '|';
        dummy += Modulation        + '|';
        dummy += CodeRL            + '|';
        dummy += Bandwidth         + '|';
        dummy += TransmissionMode  + '|';
        dummy += Guard             + '|';
      end;
      dummy += IntToStr(i+1)+'|||';
      Writeln(outf, dummy);
    end;
  end;
  CloseFile(outf);
end;


initialization
  FillSatNames;

finalization
  SatNames.Free;
  ChanNames.Free;


end.


