
//
// cle4vdr 2005 - 2010 by Alexander Hans <cle4vdr@arcor.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
// Or, point your browser to http://www.gnu.org/copyleft/gpl.html
//


unit usort;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, StrUtils;

type

  { TfrmSort }

  TfrmSort = class(TForm)
    btOK: TButton;
    btCancel: TButton;
    cbTV: TCheckBox;
    cbRadio: TCheckBox;
    cbData: TCheckBox;
    cbFTA: TCheckBox;
    cbSat: TCheckBox;
    cbProvider: TCheckBox;
    cbType: TCheckBox;
    cbGroupsep: TCheckBox;
    cbFrequency: TCheckBox;
    gbImport: TGroupBox;
    gbGroupBy: TGroupBox;
    lbGroupSep: TLabel;
    rgSortBy: TRadioGroup;
    procedure btCancelClick(Sender: TObject);
    procedure btOKClick(Sender: TObject);
    procedure cbFrequencyChange(Sender: TObject);
    procedure cbProviderChange(Sender: TObject);
  private
    { private declarations }
    confirmed: Boolean;
    TakeTV, TakeRadio, TakeData, TakeFTA: Boolean;
    GroupByProvider, GroupBySat, GroupByType, GroupByFrequency: Boolean;
    Sort : Boolean;
    AddGroupSeps: Boolean;
    procedure Parse(s: String);
    function isValidChan(s: String): Boolean;
    procedure AddToGroupArray(s: String);
    function GetSatIndex(s: String): Integer;
    function GetSatName(s: String): String;
    function GetGroupIndex(a: Integer; s: String): Integer;
    function GetGroupName(s: String): String;
    function isTVStr(s: String): Boolean;
    function isRadioStr(s: String): Boolean;
    procedure AddGroupedChannel(a, p, t: Integer; s: String);
    procedure BuildTree;
    function AddTypeGroup(t: Integer): Boolean;
    function AddProviderGroup(a, p, m: Integer): Boolean;
    function isEmptyType(a, m: Integer): Boolean;
    procedure SortSatArray;
    procedure SortChannelNames;
    procedure CleanUp;
  public
    { public declarations }
    procedure SortData(p: Pointer);
    function GetType(s: String): Integer;
    function isEncryptedStr(s: String): Boolean;
  end; 

type
  TSortArray = packed Record
    SortItem: String;
    Channels: TStringList;
  end;

type
  TGroupArray = packed Record
    GroupItem: String;
    Channels: array[0..2] of TStringList;  // 0-TV/All, 1-Radio, 2-Data
  end;


type TSatArray = packed Record
       SatName: String;
       Groups: array of TGroupArray;
     end;

var
  frmSort: TfrmSort;
  SatArray: array of TSatArray;
  ChannelType: array[0..2] of String = ('Television', 'Radio', 'Data');


implementation

{ TfrmSort }

uses uchannels;

procedure TfrmSort.btCancelClick(Sender: TObject);
begin
  confirmed := false;
  frmSort.Close;
end;

procedure TfrmSort.btOKClick(Sender: TObject);
begin
  confirmed := true;
  TakeTV     := cbTV.Checked;
  TakeRadio  := cbRadio.Checked;
  TakeData   := cbData.Checked;
  TakeFTA    := cbFTA.Checked;
  Sort       := rgSortBy.ItemIndex <> 1;
  GroupByProvider := cbProvider.Checked;
  GroupBySat      := cbSat.Checked;
  GroupByFrequency := cbFrequency.Checked;
  AddGroupSeps    := cbGroupSep.Checked;
  GroupByType     := cbType.Checked;
  frmSort.Close;
end;

procedure TfrmSort.cbFrequencyChange(Sender: TObject);
begin
  if cbFrequency.Checked then cbProvider.Checked := false;
end;

procedure TfrmSort.cbProviderChange(Sender: TObject);
begin
  if cbProvider.Checked then cbFrequency.Checked := false;
end;

procedure TfrmSort.SortData(p: Pointer);
var
  dummy: String;
begin
  if P <> NIL then
  begin
    confirmed := false;
    frmSort.ShowModal;
    if NOT confirmed then Exit;
    While Not EOF(TextFile(p^)) do
    begin
      Readln(TextFile(p^), dummy);
      Parse(dummy);
    end;
  end;
  if Sort then
  begin
    SortSatArray;
    SortChannelNames;
  end;
  BuildTree;
  CleanUp;
end;

procedure TfrmSort.Parse(s: String);
begin
  if (trim(s) <> '') AND (s[1] <> ':') AND (isValidChan(s)) then
     AddToGroupArray(s)
end;

procedure TfrmSort.AddToGroupArray(s: String);
var
  GroupItem: String;
  a, p, t: Integer;
begin
  if GroupBySat then a := GetSatIndex(s)
  else
  begin
    SetLength(SatArray, 1);
    SatArray[0].SatName := '';
    a := 0;
  end;
  if (GroupByProvider OR GroupByFrequency) then p := GetGroupIndex(a, s)
  else
  begin
    if length(SatArray[a].Groups) = 0 then
    begin
      SetLength(SatArray[a].Groups, 1);
      SatArray[a].Groups[0].Channels[0] := TStringList.Create;
      SatArray[a].Groups[0].Channels[1] := TStringList.Create;
      SatArray[a].Groups[0].Channels[2] := TStringList.Create;
    end;
    p := 0;
  end;
  if GroupByType then t := GetType(s) else t := 0;
  AddGroupedChannel(a, p, t, s);
end;

function TfrmSort.GetSatIndex(s: String): Integer;
var
  i: Integer;
  n: String;
begin
  n := GetSatName(s);
  if Length(SatArray) > 0 then
    for i := 0 to Length(SatArray)-1 do
      if SatArray[i].SatName = n then
      begin
        result := i;
        exit;
      end;
  SetLength(SatArray, length(SatArray)+1);
  SatArray[length(SatArray)-1].SatName := n;
  result := length(SatArray)-1;
end;

function TfrmSort.GetSatName(s: String): String;
begin
  result := ExtractDelimited(4, s, [':'])
end;

function TfrmSort.GetGroupIndex(a: Integer; s: String): Integer;
var
  i: Integer;
  n: String;
begin
  n := GetGroupName(s);
  if Length(SatArray) >= a-1 then
    if length(SatArray[a].Groups) > 0 then
      for i := 0 to Length(SatArray[a].Groups)-1 do
        if SatArray[a].Groups[i].GroupItem = n then
        begin
          result := i;
          exit;
      end;
  SetLength(SatArray[a].Groups, length(SatArray[a].Groups)+1);
  SatArray[a].Groups[length(SatArray[a].Groups)-1].GroupItem := n;
  SatArray[a].Groups[length(SatArray[a].Groups)-1].Channels[0] := TStringList.Create;
  SatArray[a].Groups[length(SatArray[a].Groups)-1].Channels[1] := TStringList.Create;
  SatArray[a].Groups[length(SatArray[a].Groups)-1].Channels[2] := TStringList.Create;
  result := length(SatArray[a].Groups)-1;
end;

function TfrmSort.GetGroupName(s: String): String;
begin
  if GroupByProvider then
  begin
    result := ExtractDelimited(1, s, [':']);
    if AnsiContainsStr(result, ';') then result := copy(result, pos(';', result)+1, length(result))
    else result := 'NONE'
  end
  else result := ExtractDelimited(2, s, [':']);
end;

function TfrmSort.GetType(s: String): Integer;
begin
  if isTVStr(s) then result := 0
  else if isRadioStr(s) then result := 1
  else result := 2;
end;

procedure TfrmSort.AddGroupedChannel(a, p, t: Integer; s: String);
begin
   SatArray[a].Groups[p].Channels[t].Add(s);
end;


procedure TfrmSort.BuildTree;
var
  i, j, k, l, m, currIndex: Integer;
begin
  currIndex := 0;
  if GroupByType then l := 2 else l := 0;
  for m := 0 to l do
  begin
    if (AddGroupSeps AND AddTypeGroup(m)) then AddGroupSepStr(': ---- ' + ChannelType[m] + ' ----');
    for i := 0 to length(SatArray) -1 do
    begin
      if (AddGroupSeps AND GroupBySat AND NOT isEmptyType(i, m)) then AddGroupSepStr(': ---- ' + SatArray[i].SatName + ' ----');
      for j := 0 to length(SatArray[i].Groups) -1 do
        begin
          if AddProviderGroup(i, j, m) then AddGroupSepStr(':'+SatArray[i].Groups[j].GroupItem);
          for k := 0 to SatArray[i].Groups[j].Channels[m].Count - 1 do
          begin
            AddChannelStr(SatArray[i].Groups[j].Channels[m][k]);
            inc(currIndex);
          end;
        end;
    end;
  end;
end;

function TfrmSort.AddTypeGroup(t: Integer): Boolean;
var
  i, j: Integer;
begin
  result := false;
  Case t of
    0: if NOT TakeTV then exit;
    1: if NOT TakeRadio then exit;
    2: if NOT TakeData then exit;
  end;
  if NOT GroupByType then exit
  else
  begin
    for i := 0 to length(SatArray)-1 do
      for j := 0 to length(SatArray[i].Groups) -1 do
        if SatArray[i].Groups[j].Channels[t].Count > 0 then
        begin
          result := true;
          exit;
        end;
  end;
end;

function TfrmSort.AddProviderGroup(a, p, m: Integer): Boolean;
begin
  if (GroupByProvider OR GroupByFrequency) AND
     (SatArray[a].Groups[p].Channels[m].Count > 0) then result := true
  else result := false;
end;

function TfrmSort.isEmptyType(a, m: Integer): Boolean;
var
  i: Integer;
begin
  for i := 0 to length(SatArray[a].Groups)-1 do
    if SatArray[a].Groups[i].Channels[m].Count > 0 then
    begin
      result := false;
      exit;
    end;
  result := true;
end;

// *********** Helper Functions ***********//


function TfrmSort.isEncryptedStr(s: String): Boolean;
begin
  result := ExtractDelimited(9, s, [':']) <> '0';
end;

function TfrmSort.isTVStr(s: String): Boolean;
begin
  result := ExtractDelimited(6, s, [':']) <> '0';
end;

function TfrmSort.isRadioStr(s: String): Boolean;
begin
  result := ((ExtractDelimited(6, s, [':']) = '0')
        AND (ExtractDelimited(7, s, [':']) <> '0'))
end;


function TfrmSort.isValidChan(s: String): Boolean;
begin
  result := false;
  if (TakeFTA AND isEncryptedStr(s)) then
  begin
    result := false;
    exit;
  end;
  if (TakeTV AND isTVStr(s)) then
  begin
    result := true;
    exit;
  end;
  if (TakeRadio AND isRadioStr(s)) then
  begin
    result := true;
    exit;
  end;
  if (TakeData AND (NOT isTVStr(s)) AND (NOT isRadioStr(s))) then
  begin
    result := true;
    exit;
  end
end;


procedure TfrmSort.SortSatArray;
var
  i,j, n, a : integer;
  dummy: TGroupArray;
begin
  if Length(SatArray) = 0 then Exit;
  for a := 0 to Length(SatArray) -1 do
  begin
    n := length(SatArray[a].Groups);
    for i := 0 to n-2 do
      for j := i+1 to n-1 do
       if SatArray[a].Groups[i].GroupItem > SatArray[a].Groups[j].GroupItem then
       begin
         dummy := SatArray[a].Groups[i];
         SatArray[a].Groups[i] := SatArray[a].Groups[j];
         SatArray[a].Groups[j] := dummy
      end
  end
end;

procedure TfrmSort.SortChannelNames;
var
  i, j, k: Integer;
begin
  for i := 0 to length(SatArray) -1 do
    for j := 0 to length(SatArray[i].Groups) - 1 do
      for k := 0 to 2 do
        SatArray[i].Groups[j].Channels[k].Sort;
end;


procedure TfrmSort.CleanUp;
var
  i, j, k: Integer;
begin
  for i := 0 to length(SatArray) -1 do
  begin
    for j := 0 to length(SatArray[i].Groups) - 1 do
    begin
      for k := 0 to 2 do
      begin
        SatArray[i].Groups[j].Channels[k].Clear;
        SatArray[i].Groups[j].Channels[k] := NIL;
      end;
    end;
    SetLength(SatArray[i].Groups, 0);
  end;
  SetLength(SatArray, 0);
end;




initialization
  {$I usort.lrs}

end.

