
//
// cle4vdr 2005 - 2010 by Alexander Hans <cle4vdr@arcor.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
// Or, point your browser to http://www.gnu.org/copyleft/gpl.html
//


unit uupdate;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, StrUtils;

type

  { TfrmUpdate }

  TfrmUpdate = class(TForm)
    btnCancel: TButton;
    btnOK: TButton;
    cgType: TCheckGroup;
    cbDelete: TCheckBox;
    cbLog: TCheckBox;
    pbUpdate: TProgressBar;
    rgPay: TRadioGroup;
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    LogStr: TStringList;
    procedure AssignLog;
    procedure UpdateLog(s: String);
    procedure WriteLog;
    procedure UpdateList(path: String; t, r, o, d, p: Boolean);
  public
    { public declarations }
    procedure InitUpdate(path: String);
  end; 

var
  frmUpdate: TfrmUpdate;


procedure UpdateParse(s, p: String; var sl: TStringList; t, r, o, f: Boolean);
procedure BuildIDList(var idl: TStringList);
function ChannelIDfromStr(s: String): String;
procedure UpdateChannels(sorted: Boolean);
procedure RemoveChannels(var idl: TStringList);
function GetBouquet(s: String): TTreeNode;
procedure CleanUp;

var
  NewChannels: TStringList;

implementation

uses uinterface, uchannels, uutils, uinfo, uedit, usort;


procedure TfrmUpdate.InitUpdate(path: String);
begin
  frmUpdate.ShowModal;
  if frmUpdate.ModalResult = 1 then UpdateList(path, cgType.Checked[0],
                                 cgType.Checked[1], cgType.Checked[2],
                                 cbDelete.Checked, rgPay.ItemIndex = 1);
  frmUpdate.Visible:=false;
end;

procedure TfrmUpdate.FormCreate(Sender: TObject);
begin
  cgType.Checked[0] := true;
  cgType.Checked[1] := true;
end;

procedure TfrmUpdate.AssignLog;
begin
  LogStr := TStringList.Create;
end;


procedure TfrmUpdate.UpdateLog(s: String);
begin
  LogStr.Add(s);
end;

procedure TfrmUpdate.WriteLog;
begin
  if frmUpdate.cbLog.Checked then
  begin
    try
      if LogStr.Count = 2 then LogStr.Add('no updates found');
      LogStr.SaveToFile(GetTempDir(false) + 'cleout.txt');
      showmessage('Log saved to ' + GetTempDir(false) + 'cleout.txt');
    Except
      showmessage('Failed to write log to ' + GetTempDir(false));
    end;
  end;
  LogStr.Free;
end;

procedure TfrmUpdate.UpdateList(path: String; t, r, o, d, p: Boolean);
var
  inf: TextFile;
  dummy, bouq: String;
  IDList: TStringList;
begin
  AssignLog;
  UpdateLog('Updating from ' + path);
  UpdateLog('');
  AssignFile(inf, path);
  Reset(inf);
  IDList := TStringList.Create;
  BuildIDList(IDList);
  NewChannels := TStringList.Create;
  bouq := '';
  frmMain.tvMain.BeginUpdate;
  While NOT EOF(inf) do
  begin
    Readln(inf, dummy);
    if dummy = '' then continue;
    if dummy[1] = ':' then bouq := copy(dummy, 2, length(dummy))
    else UpdateParse(dummy, bouq, IDList, t, r, o, p);
  end;
  CloseFile(inf);
  UpdateChannels(true);
  if d then RemoveChannels(IDList);
  IDList.Free;
  NewChannels.Free;
  CleanUp;
  frmUpdate.WriteLog;
  RenumberChannels;
  frmMain.tvMain.EndUpdate;
  frmMain.RefreshTwinView;
  sbm(GetChannelsInfo, 1);
end;

function UpdateThis(s: String; t, r, o, f: Boolean): Boolean;
var
  i: Integer;
begin
  result := false;
  i := frmSort.GetType(s);
  if (i = 0) AND (NOT t) then Exit;
  if (i = 1) AND (NOT r) then Exit;
  if (i = 2) AND (NOT o) then Exit;
  if (NOT f) AND (frmSort.isEncryptedStr(s)) then exit;
  result := true;
end;


procedure UpdateParse(s, p: String; var sl: TStringList; t, r, o, f: Boolean);
var
  I: Integer;
begin
  i := sl.IndexOf(ChannelIDfromStr(s));
  if i = -1 then
  begin
    if NOT UpdateThis(s, t, r, o, f) then Exit;
    s := AnsiReplaceText(s, '=', '§§');
    NewChannels.Add(s+'='+p);
  end
  else sl.Delete(i);;
end;

procedure BuildIDList(var idl: TStringList);
var
  i: Integer;
begin
  if frmMain.tvMain.Items.Count = 0 then Exit;
  for i := 0 to frmMain.tvMain.Items.Count - 1 do
  begin
    if NOT isGroupSep(frmMain.tvMain.Items[i]) then
       idl.AddObject(ChannelID(frmMain.tvMain.Items[i]), frmMain.tvMain.Items[i]);
  end;
end;

function ChannelIDfromStr(s: String): String;
var
  sl: TStringList;
  pol: String;
begin
  result := '';
  sl := StringSplit(s, ':');
  result += sl[3] + '-';
  result += sl[10] + '-';
  if (sl[11] = '0') AND (sl[10] = '0') then
  begin
    if sl[2] <> '' then
    begin
      if sl[3] = 'P' then Pol := sl[2]
      else Pol := sl[2][1]
    end
    else Pol := '';
    result += ChanIdFreq(Pol, sl[1]) + '-'
  end
  else result += sl[11] + '-';
  result += sl[9] + '-';
  if sl.Count > 12 then
      result += sl[12]
  else result += '0';
  sl.Free;
end;

procedure UpdateChannels(sorted: Boolean);
var
  i: Integer;
  n: TTreeNode;
  dummy, oldbouqet: String;
begin
  if NewChannels.Count = 0 then Exit;
  AddGroupSepStr(':Updated');
  n := NIL;
  oldbouqet := '';
  for i := 0 to NewChannels.Count - 1 do
  begin
    if sorted then
      if oldbouqet <> NewChannels.ValueFromIndex[i] then
        n := GetBouquet(NewChannels.ValueFromIndex[i]);
    oldbouqet := NewChannels.ValueFromIndex[i];
    dummy := AnsiReplaceText(NewChannels.Names[i], '§§', '=');
    if n = NIL then
    begin
      frmUpdate.UpdateLog('Added ' + copy(dummy, 1, Pos(':', dummy)-1) + ' at Updated');
      AddChannelStr(dummy)
    end
    else
    begin
      frmUpdate.UpdateLog('Added ' + copy(dummy, 1, Pos(':', dummy)-1) + ' at ' + n.Text);
      InsertChannel(dummy, n, false);
    end;
  end;
end;

procedure RemoveChannels(var idl: TStringList);
var
  i: Integer;
begin
  if idl.Count = 0 then Exit;
  for i := 0 to idl.Count - 1 do
  begin
    frmUpdate.UpdateLog('Deleting ' + TTreeNode(idl.Objects[i]).Text + ' (' + idl[i] + ')');
    DeleteNode(frmMain.tvMain, TTreeNode(idl.Objects[i]), false);
  end;
end;

function GetBouquet(s: String): TTreeNode;
var
  i: Integer;
  bouq: String;

begin
  result := NIL;
  s := AnsiReplaceStr(s, '->', '');
  for i := 0 to frmMain.tvMain.Items.Count - 1 do
  begin
    if (frmMain.tvMain.Items[i].Level = 0) AND (isGroupSep(frmMain.tvMain.Items[i])) then
      if trim(s) = trim(cleP(frmMain.tvMain.Items[i].Data)^.Name) then
      begin
        result := frmMain.tvMain.Items[i];
        Exit;
      end;
  end;
end;

procedure CleanUp;
var
  n: TTreeNode;
begin
  n := frmMain.tvMain.Items[frmMain.tvMain.Items.Count-1];
  if (n <> NIL) AND (n.Text = 'Updated') AND (NOT n.HasChildren) then
      DeleteNode(frmMain.tvMain, n, false);
end;



initialization
  {$I uupdate.lrs}

end.

