
//
// cle4vdr 2005 - 2010 by Alexander Hans <cle4vdr@arcor.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
// Or, point your browser to http://www.gnu.org/copyleft/gpl.html
//


unit uutils;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Dialogs, StdCtrls, ComCtrls, StrUtils;

function StringSplit(const splitstring:String; const SearchChar:Char): TStringList;
function LTrunc(const str: String; const c: Integer): String;
function RTrunc(const str: String; const c: Integer): String;
function StripTrailingChar(const s: String; const achar: TSysCharSet): String;
function SetStrLen(const s: String; const len: Integer): String;
function InsertText(str, insert: String; i: Integer): String;
function StartsWith(s: String; part: String): Boolean;


function TryInteger(const new: String; var target: String): Boolean;
//function TryPol(const new: String; var target: String): Boolean;
function TryCa(const new: String; var target: String): Boolean;
function TrySource(const new: String; var target: String): Boolean;
//function TryAudPid(new, old: String; shmes: Boolean; ed: TEdit): String;
function TryJumpMark(new: String): Boolean;


function SetLastTarget(n: TTreeNode): TTreeNode;
function isFirstChild(n: TTreeNode): Boolean;

procedure SavePrefs(s, o: String);
procedure ReadPrefs(var s, o: String);


implementation

uses uchannels, uinfo, uinterface;

// ********** Str Utils ************//

function StringSplit(const splitstring:String; const SearchChar:Char): TStringList;
var
tempstr : AnsiString;
i : Integer;
begin
  tempstr := '';
  result := TStringlist.Create;
  for i:= 1 to Length(splitstring) do
  begin
    if splitstring[i] <> SearchChar then
      tempstr := tempstr + splitstring[i]
    else
    begin
     if tempstr<>'' then
     begin
      result.Add(tempstr);
      tempstr := '';
     end
     else if tempstr = '' then
     begin
       tempstr := ' ';
       result.Add(tempstr);
       tempstr := '';
     end;
    end;
   end;
  result.Add(tempstr);
end;

function LTrunc(const str: String; const c: Integer): String;
begin
  result := str;
  if length(str) >= c then
    result := copy(str, c+1, length(str)-c);
end;

function RTrunc(const str: String; const c: Integer): String;
begin
  result := str;
  if length(str) >= c then
    result := copy(str, 1, length(str)-c);
end;


function StripTrailingChar(const s: String; const achar: TSysCharSet): String;
begin
  result := s;
  if length(s) > 0 then
  begin
    if s[1] in achar then
    begin
      if length(s) = 1 then
        result := ''
      else
      begin
        result := trim(copy(s, 2, length(s)));
        result := StripTrailingChar(result, achar);
      end;
    end;
  end;
end;


function SetStrLen(const s: String; const len: Integer): String;
var
  i: Integer;
begin
  i := 1;
  if Length(s) <= len then result := s
  else
  begin
    while i < len do
    begin
      result += s[i];
      inc(i);
    end;
    result += '..';
  end;
end;


function InsertText(str, insert: String; i: Integer): String;
begin
  result := copy(str, 1, i);
  result += insert;
  result += copy(str, i+1, length(str));
end;

function StartsWith(s: String; part: String): Boolean;
var
  i: Integer;
begin
  result := false;
  if length(part) <= length(s) then
  begin
    for i := 1 to length(part) do
      if part[i] <> s[i] then Exit;
    result := true;
  end;
end;


// ********** Conversion *************//

function TryInteger(const new: String; var target: String): Boolean;
var
  i: Integer;
begin
  result := TryStrToInt(new, i);
  if result = true then target := new;
end;


function TryHex(s: String): Boolean;
begin
  Try
    Result := true;
    StrToInt('$'+trim(s));
  Except
    on eConvertError do result := false;
  end;
end;

function TryCa(const new: String; var target: String): Boolean;
var
  strl: TStringList;
  i: Integer;
begin
  strl := StringSplit(new, ',');
  result := true;
  for i := 0 to strl.Count -1 do
  begin
    if NOT TryHex(strl[i]) then
    begin
      result := false;
      Break;
    end;
  end;
  strl.Free;
  if result = true then target := UpperCase(new);
end;

function TrySource(const new: String; var target: String): Boolean;
begin
  if not AnsiStartsText('S', new) then target := 'S'+new;
  result := true;
end;


function TryJumpMark(new: String): Boolean;
begin
  result := GetJumpIndex(':' + new) <> -1;
end;


function SetLastTarget(n: TTreeNode): TTreeNode;
begin
  if isGroupSep(n) then result := n
  else result := n.Parent;
end;

function isFirstChild(n: TTreeNode): Boolean;
begin
  result := false;
  if (n = NIL) OR (isGroupSep(n)) OR (n.Parent = NIL) then Exit;
  if n.Parent.GetFirstChild = n then result := true
end;

procedure SavePrefs(s, o: String);
var
  outf: Textfile;
begin
  try
    if not DirectoryExists(GetAppConfigDir(false)) then CreateDir(GetAppConfigDir(false));
    AssignFile(outf, GetAppConfigDir(false) + 'cle4vdr.conf');
    Rewrite(outf);
    Writeln(outf, 'OpenPath='+o);
    Writeln(outf, 'SavePath='+s);
    CloseFile(outf);
  Except
    Exit;
  end;
end;

procedure ReadPrefs(var s, o: String);
var
  conf: TStringList;
begin
  try
    conf := TStringList.Create;
    conf.LoadFromFile(GetAppConfigDir(false) + 'cle4vdr.conf');
    s := conf.Values['SavePath'];
    o := conf.Values['OpenPath'];
    conf.Free;
  Except
    Exit;
  end;
end;

end.

